from __future__ import annotations

import typing_extensions as te

if te.TYPE_CHECKING:
    import uuid

import click

from gcs_cli.helpers import record_print_result_message, requires_login
from gcs_cli.parsing import globus_command
from gcs_cli.sdk_client.data import BatchDocument
from gcs_cli.services.gcs import get_client

MAX_COLLECTION_COUNT = 50


def _dynamic_doc_string(doc_string: str) -> te.Callable:
    """
    Decorator to insert a dynamic __doc__. In Python, __doc__ can not be dynamic
    otherwise.
    """

    def _wrapper(f: te.Callable) -> te.Callable:
        f.__doc__ = doc_string
        return f

    return _wrapper


@globus_command(
    "batch-delete",
    short_help="Delete multiple guest collections",
    disable_options=["format"],
)
@click.argument(
    "collection_id",
    metavar="COLLECTION_ID [COLLECTION_ID...]",
    type=click.UUID,
    nargs=-1,
    required=True,
)
@requires_login
@_dynamic_doc_string(
    f"""
    Delete existing collections. This requires the administrator role on the
    Endpoint. Max of {MAX_COLLECTION_COUNT} collections
    """
)
def collection_batch_delete(collection_id: tuple[uuid.UUID], **kwargs: te.Any) -> None:
    if len(collection_id) > MAX_COLLECTION_COUNT:
        raise click.UsageError(f"Maximum batch size of {MAX_COLLECTION_COUNT}")
    client = get_client()
    data = BatchDocument(ids=collection_id)
    res = client.batch_collection_delete(data=data)
    record_print_result_message(res)
