from __future__ import annotations

import typing as t
import uuid

from globus_sdk import MISSING, EndpointDocument

from gcs_cli.helpers import requires_login, validate_network_use
from gcs_cli.parsing import (
    EXPLICIT_NULL,
    ExplicitNullClass,
    endpoint_update_params,
    globus_command,
)
from gcs_cli.prettyprint import FORMAT_TEXT_RECORD, formatted_print
from gcs_cli.services.gcs import get_sdk_client


def sdk_convert_param(p: t.Any) -> t.Any:
    """
    Converts a click parameter's value from CLI-internal types to SDK-internal
    types.

    None -> MISSING
    EXPLICIT_NULL -> None
    """
    if p is EXPLICIT_NULL:
        return None
    if p is None:
        return MISSING
    return p


@globus_command("update", short_help="Update an endpoint definition")
@endpoint_update_params
@requires_login
def endpoint_update(
    display_name: str | None = None,
    public: bool | None = None,
    description: str | ExplicitNullClass | None = None,
    info_link: str | ExplicitNullClass | None = None,
    contact_info: str | ExplicitNullClass | None = None,
    contact_email: str | ExplicitNullClass | None = None,
    organization: str | ExplicitNullClass | None = None,
    department: str | ExplicitNullClass | None = None,
    keywords: list[str] | None = None,
    subscription_id: uuid.UUID | str | ExplicitNullClass | None = None,
    network_use: t.Literal["normal", "minimal", "aggressive", "custom"] | None = None,
    max_concurrency: int | None = None,
    preferred_concurrency: int | None = None,
    max_parallelism: int | None = None,
    preferred_parallelism: int | None = None,
    gridftp_control_channel_port: tuple[int, int] | None = None,
) -> None:
    """
    Update the endpoint which the GCS CLI is logged into
    """

    if isinstance(subscription_id, uuid.UUID):
        subscription_id = str(subscription_id)

    doc = EndpointDocument(
        display_name=sdk_convert_param(display_name),
        public=sdk_convert_param(public),
        description=sdk_convert_param(description),
        info_link=sdk_convert_param(info_link),
        contact_info=sdk_convert_param(contact_info),
        contact_email=sdk_convert_param(contact_email),
        organization=sdk_convert_param(organization),
        department=sdk_convert_param(department),
        keywords=sdk_convert_param(keywords),
        subscription_id=sdk_convert_param(subscription_id),
        network_use=sdk_convert_param(network_use),
        max_concurrency=sdk_convert_param(max_concurrency),
        preferred_concurrency=sdk_convert_param(preferred_concurrency),
        max_parallelism=sdk_convert_param(max_parallelism),
        preferred_parallelism=sdk_convert_param(preferred_parallelism),
        gridftp_control_channel_port=sdk_convert_param(gridftp_control_channel_port),
    )

    # Temporary casting until all uses of validate_network_use() are using the
    # SDK data doc types
    validate_network_use(
        {
            "network_use": network_use,
            "max_concurrency": max_concurrency,
            "preferred_concurrency": preferred_concurrency,
            "max_parallelism": max_parallelism,
            "preferred_parallelism": preferred_parallelism,
        }
    )

    client = get_sdk_client()
    res = client.update_endpoint(endpoint_data=doc)

    formatted_print(res, fields=[("code", "code")], text_format=FORMAT_TEXT_RECORD)
