"""
Define command decorators which set requirements on caller's logged-in status before
running the command.
"""
from functools import partial, wraps
from typing import Any, Callable, List, Optional

import click

from gcs_cli.config import GCS_CLIENT_DATA_CONFIG_KEY, token_storage_adapter

from .client_login import CLIENT_ENV_VARS_STRING, is_client_login
from .parser_hook import parser_hook


def is_logged_in() -> bool:
    adapter = token_storage_adapter()
    return bool(adapter.read_config(GCS_CLIENT_DATA_CONFIG_KEY))


def requires_login(command: Callable) -> Callable:
    """
    Decorate a sub command to verify that the caller is logged in.

    The caller can use either an access token provided by an earlier call to
    'login' or the env can be populated with client credentials.

    is_client_login() has a side effect of raising an exception if the client env
    is only partially set.
    """

    @wraps(command)
    def wrapped(*args: Any, **kwargs: Any) -> Any:
        if not is_client_login() and not is_logged_in():
            raise click.ClickException("You must log in before running this command.")
        return command(*args, **kwargs)

    return wrapped


def _lookup_cmd_name(ctx: click.Context) -> str:
    """
    Given the current click context, return the sub command as a string.
    """
    subcmds: List[str] = []
    _ctx: Optional[click.Context] = ctx
    while _ctx is not None:
        # mypy whining
        assert isinstance(_ctx.command.name, str), "Command name is not a string"
        subcmds.insert(0, _ctx.command.name)
        _ctx = _ctx.parent
    return " ".join(subcmds)


_DENY_CLIENT_LOGIN_MESSAGE = f"""\
{CLIENT_ENV_VARS_STRING} are set.

When using client credentials, do not use '{{}}'
"""


def deny_client_login(
    helpful_error_msg: Optional[str] = None,
) -> Callable[[Callable], Callable]:
    """
    Decorator that will deny use of client logins for decorated commands.

    This decorator will raise click.UsageError if one or more client env vars are set.
    """

    def _callback() -> None:
        if is_client_login(true_if_partial=True):
            cmd_name = _lookup_cmd_name(click.get_current_context())
            raise click.UsageError(
                f"{_DENY_CLIENT_LOGIN_MESSAGE}"
                f"{helpful_error_msg if helpful_error_msg else ''}".format(cmd_name)
            )

    return partial(parser_hook, "disable_client_login", _callback)
