from __future__ import annotations

from functools import wraps

import click
import typing_extensions as te

T = te.TypeVar("T")


def parser_hook(
    option_name: str,
    callback: te.Callable[[], None],
    command: te.Callable[..., T],
) -> te.Callable:
    """
    Add a hook to call 'callback' before command line processing begins.

    For a normal function decorator, Click will process options and arguments
    before calling 'command'. In that case, the decorator would not be able to
    deny client login attempts until after the caller has the correct command
    syntax.  So instead, we add a fake/hidden option and hook its call to the
    start of the syntax parsing process and add our check there.

    Our hook is implemented as a hidden option which is then removed on success
    before the actual function is called.
    """

    class _OptionParserHook(click.Option):
        """
        Subclass click.Option so that we can capture the call to start processing
        command line options.
        """

        def add_to_parser(self, *args: te.Any, **kwargs: te.Any) -> None:
            callback()
            super().add_to_parser(*args, **kwargs)

    @wraps(command)
    def _remove_hidden_option(*args: te.Any, **kwargs: te.Any) -> T:
        """
        Remove our hidden function arg before calling the actual function.
        """
        kwargs.pop(option_name.replace("-", "_"))
        return command(*args, **kwargs)

    return click.option(
        f"--{option_name.replace('_', '-')}",
        is_flag=True,
        hidden=True,
        cls=_OptionParserHook,
    )(_remove_hidden_option)
