from __future__ import annotations

import ipaddress

import ifaddr
import requests


def is_ec2() -> bool:
    """
    Returns True if the current machine appears to be an ec2 instance, False
    otherwise.
    """
    # This is based on the documentation from amazon here:
    # https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/identify_ec2_instances.html
    result = False
    hypervisor_uuidfile = "/sys/hypervisor/uuid"
    try:
        with open(hypervisor_uuidfile, "r") as f:
            result = f.read(3).lower() == "ec2"
    except Exception:
        pass

    if not result:
        # Check this for both big-endian and little-endian uuids
        # Only works when run as root, but is ok for node setup
        dmi_product_uuid_file = "/sys/devices/virtual/dmi/id/product_uuid"
        try:
            with open(dmi_product_uuid_file, "r") as f:
                uuid = f.read(36).lower()
                result = uuid[0:3] == "ec2" or (uuid[6:8] + uuid[4]) == "ec2"
        except Exception:
            pass

    return result


def public_ip() -> ipaddress.IPv4Address | ipaddress.IPv6Address | None:
    """
    Determine a public IP address for this machine. This employs two different
    algorithms:

    *  If this is an EC2 instance, parse the metadata and extract the public
       IPv4 address
    *  Enumerate network interfaces and check for a single public IP address
    """
    value: ipaddress.IPv4Address | ipaddress.IPv6Address | None = None
    valuestr: str | None = None
    valuestr_v6: str | None = None

    if is_ec2():
        ip_addrs = ["169.254.169.254", "[fd00:ec2::254]"]
        headers = {}
        for addr in ip_addrs:
            token_url = f"http://{addr}/latest/api/token"
            try:
                res = requests.put(
                    token_url,
                    headers={"X-aws-ec2-metadata-token-ttl-seconds": "30"},
                    timeout=3.0,
                )
                if res.status_code == 200:
                    headers = {"X-aws-ec2-metadata-token": res.text.strip()}
                    break
            except IOError:
                pass

        for addr in ip_addrs:
            url = f"http://{addr}/latest/meta-data/public-ipv4"
            try:
                res = requests.get(url, headers=headers, timeout=3.0)
                if res.status_code == 200:
                    valuestr = res.text.strip()
                    if valuestr != "":
                        break
            except IOError:
                pass

        if valuestr == "":
            for addr in ip_addrs:
                url = f"http://{addr}/latest/meta-data/ipv6"
                try:
                    res = requests.get(url, headers=headers, timeout=3.0)
                    if res.status_code == 200:
                        valuestr_v6 = res.text.strip()
                        if (
                            valuestr_v6 != ""
                            and ipaddress.ip_address(valuestr).is_global
                        ):
                            break
                        else:
                            valuestr_v6 = ""
                except IOError:
                    pass

    if valuestr:
        ipvalue = ipaddress.ip_address(valuestr)
        value = ipvalue
    elif valuestr_v6:
        ipvalue = ipaddress.ip_address(valuestr_v6)
        value = ipvalue

    if value is None:
        # Enumerate network adapters and check for a public IPv4 address
        adapters = ifaddr.get_adapters()

        public_addresses: list[ipaddress.IPv4Address] = []

        for adapter in adapters:
            for ip in adapter.ips:
                if ip.is_IPv4:
                    # Let mypy know this is type str so it won't complain on
                    # ip_address()
                    assert isinstance(ip.ip, str)
                    address = ipaddress.ip_address(ip.ip)
                    assert isinstance(address, ipaddress.IPv4Address)
                    if address.is_global:
                        public_addresses.append(address)
        if len(public_addresses) == 1:
            value = public_addresses[0]

    return value
